#!/bin/sh
# FILE: oommf.tcl
#
# This Tcl script is the master application of the OOMMF project.
# It must be evaluted by a Tcl interpreter such as tclsh or wish
# (or whatever other name they are known by on your platform).  
#
# On Unix platforms, it may be necessary to edit line 17 of this file
# so that the second word is the name of a Tcl interpreter shell 
# installed on your computer. 
#
# On Windows platforms, when Tcl/Tk was installed, an association was
# placed in the registry to recognize the file extension .tcl of this
# file and use the installed wish program to evaluate it.
#
#    v--Edit here if necessary \
exec tclsh "$0" ${1+"$@"}

set oommf(version) 1.1.0.0
set oommf(defaultArgv) mmLaunch
set oommf(usage) [subst {
Usage:
-----
oommf.tcl ?options? ?app-spec? ?args?
	where
Valid options:
	-fg	Run app in foreground; block till it completes
	-bg	Run app in background; exit after starting it (default)
	-command	Print app's command line to stdout and quit
	-version	Print version of oommf.tcl and quit
	-platform	Print platform name and quit
	-h*	Print usage summary and quit
	--	End of options, next argument is app-spec
Valid app-spec: 'appName ?-exact? ?version-requirement?'
	(NB: app-spec is quoted as a single argument)
	(NB2: Default is '$oommf(defaultArgv)'.)
	where
		appName			Name of app to start (case insensitive)
		-exact			When present, version-requirement
					is exact; later releases are not
					acceptable
		version-requirement	Minimum (or exact) version which
					is acceptable.  When omitted,
					accept latest version available.
?args?
	rest of command line is passed as command line arguments
	of the application started.
}]

# This whole script is wrapped in a 'catch' so we can process any errors
# in such a way that the error message is displayed to the user regardless
# of platform and Tcl version
if {[catch {

# Create search path for OOMMF apps and extensions
set oommf(libPath) {}
if {[info exists env(OOMMF_ROOT)] \
        && [string match absolute [file pathtype $env(OOMMF_ROOT)]] \
        && [file isdirectory [file join $env(OOMMF_ROOT) app]]} {
    lappend oommf(libPath) $env(OOMMF_ROOT)
}
# Get the absolute form of the directory containing this script
set cwd [pwd]
set myAbsoluteName [file join $cwd [info script]]
if {[catch {cd [file dirname $myAbsoluteName]} msg]} {
    error "" \
"Can't determine direct pathname for $myAbsoluteName:
        $msg"
}
set myAbsoluteName [file join [pwd] [file tail $myAbsoluteName]]
cd $cwd
set dir [file dirname $myAbsoluteName]
if {[file isdirectory [file join $dir app]]} {
    lappend oommf(libPath) $dir
}
if {![llength $oommf(libPath)]} {
    error "" "Unable to find the OOMMF root directory.\n\nSet the\
            environment variable OOMMF_ROOT to its absolute location."
}

# Construct search path for extensions
# Only put absolute paths on the auto_path list
foreach lib $oommf(libPath) {
    set dir [file join $lib ext]
    if {[file isdirectory $dir] && [file readable $dir] \
            && [string match absolute [file pathtype $dir]]} {
        if {[lsearch -exact $auto_path $dir] < 0} {
            lappend auto_path $dir
        }
    }
}

# Locate a version of the Oc (OOMMF core) extension and set it up for loading.
#
# The catch is here to protect against errors in the pkgIndex.tcl files
# of other packages which are not caught in pre-8.0 versions of Tcl.
if {[catch {package require Oc 1.1}]} {
    # Either we couldn't find an Oc extension, or the pkgIndex.tcl file
    # of some other package contained an error and we're running a
    # pre-8.0 version of Tcl.  In the latter case, it's possible that
    # before the error was encountered, the pkgIndex.tcl file for the
    # Oc extension was evaluated.  In that case, the workings of the
    # 'package require' command have been redefined to be robust against
    # errors in pkgIndex.tcl files.  It makes sense to try again.  If it
    # still can't find it this time, either the Oc extension really is 
    # missing, or the bad pkgIndex.tcl file is evaluated before the
    # pkgIndex.tcl file for the Oc extension, and there's nothing we can do.
    package require Oc 1.1
}

# Search relative to $oommf(libPath) for applications
Oc_Application SetSearchPath $oommf(libPath)

# Process the options
set bg 1
while {[string match -* [lindex $argv 0]]} {
    regexp -- -(.*) [lindex $argv 0] _ option
    set argv [lrange $argv 1 end]
    switch -exact -- $option {
        - {
            break
        }
	version {
	    Oc_Log Log "\nOOMMF bootstrap $oommf(version)" info
	    exit
	}
	platform {
	    Oc_Log Log "\nPlatform name:\t[[Oc_Config RunPlatform] \
                    GetValue platform_name]" info
	    exit
	}
	command {
            while {[string match -* [lindex $argv 0]]} {
                regexp -- -(.*) [lindex $argv 0] _ option
                set argv [lrange $argv 1 end]
                if {[string match - $option]} {break}
            }
            set code [catch {eval Oc_Application CommandLine $argv} msg]
            if {$code} {
                Oc_Log Log $msg panic
            } else {
                puts stdout $msg
            }
	    exit $code
	}
	fg {
	    set bg 0
	}
	bg {
	    set bg 1
	}
        default {
            if {[string match h* $option]} {
                Oc_Log Log $oommf(usage) info
                exit
            }
            Oc_Log Log "Unrecognized option: -$option\n$oommf(usage)" panic
            exit 1
        }
    }
}

# Use default if no command line
if {[llength $argv] == 0} {
    set argv $oommf(defaultArgv)
}

if {$bg} {
    if {[catch {eval Oc_Application Exec $argv &} msg]} {
        # Couldn't launch background app; report the problem
        Oc_Log Log $msg panic
	exit 1
    }
} else {
    if {[catch {eval Oc_Application CommandLine $argv} cmd]} {
        Oc_Log Log $cmd panic
        exit 1
    }
    proc FgExec {args} {
        if {![llength $args]} {return}
        foreach word $args {
            if {[set stdoutRedirected [string match >* $word]]} {break}
        }
        if {$stdoutRedirected} {
            if {[catch {eval exec <@ stdin 2>@ stderr $args} res]} {
                Oc_Log Log $res panic
                exit 1
            }
        } else {
            global FgExecDone
            set f [open [concat | <@ stdin 2>@ stderr $args] r]
            fileevent $f readable [list FgExecHandleOutput $f]
            vwait FgExecDone
            unset FgExecDone
            if {[catch {close $f} res]} {
                Oc_Log Log $res panic
                exit 1
            }
        }
    }
    proc FgExecHandleOutput {f} {
        # Blocking read -- returns -1 on EOF
        if {[gets $f line]<0} {
            global FgExecDone
            set FgExecDone 1
            return
        }
        puts $line
    }
    eval FgExec $cmd
}

} catchmsg]} {
    if {[info tclversion] < 7.5} {
        puts stderr "OOMMF software requires Tcl/Tk versions 7.5/4.1\
                or higher.\nYour version(s):\n\tTcl:\t[info patchlevel]"
        if {[info exists tk_patchLevel]} {
            puts stderr "\tTk:\t$tk_patchLevel"
        }
        puts stderr "Please upgrade."
        exit 1
    }
    # The wish shell didn't portably display errors in startup scripts to 
    # the user until Tk 8.0.  If an earlier version of Tk is evaluating 
    # this script, we have to display the message ourselves.  The 'tk_dialog'
    # command is most reliable since it's been around the longest.
    if {[info exists tk_version] && $tk_version < 8} {
        tk_dialog .error "Error in startup script" $errorInfo error 0 OK
        exit 1
    } else {
        return -code error -errorinfo $errorInfo -errorcode $errorCode ""
    }
}

# In case this script is sourced by a 'wish' shell, explicitly call 'exit'
# so we don't enter the event loop
exit 0

